using System;
using System.Drawing;

namespace TileTest
{
	/// <summary>
	/// Summary description for TileSet.
	/// </summary>
	public class TileSet
	{
		Image[] tiles;

		private const int tileSize = 48;
		private int totalTiles;
		// Tiles are 16x16
		public TileSet(string filename)
		{
			Image img = Image.FromFile(filename);
            int countx = img.Size.Width / 16;
			int county = img.Size.Height / 16;

			totalTiles = countx * county;

			tiles = new Image[totalTiles + 1];

			int cur = 0;
			
			
			
			for(int ox = 0; ox < countx; ox += 6)
			{
				for(int y = 0; y < county; y++)
				{
					for(int x = ox; x < ox + 6; x++)
					{
						Image tile = new Bitmap(tileSize, tileSize);
						Graphics g = Graphics.FromImage(tile);
						
						g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.None;
						g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.NearestNeighbor;
						g.DrawImage(img, new RectangleF(0f, 0f, tileSize, tileSize), new RectangleF(x * 16 + 0.5f, y * 16 + 0.5f, 15, 15), GraphicsUnit.Pixel);
                                        					
						tiles[cur] = tile;
						cur ++;
					}
				}
			}

		               			
		}

		public Image this [int index]
		{
			get 
			{
				return tiles[index];
			}
		}

		public int TileCount
		{
			get
			{
				return totalTiles;
			}
		}

		public int TileSize
		{
			get
			{
				return tileSize;
			}
		}
	}
}
